@@ -17,9 +17,6 @@ ERROR = "error"
 SUCCESS = "SUCCESS"
 FAILED = "FAILED"
 
-# Environment Variables
-
-
 ################################################################################
 # Queries
 ################################################################################
@@ -27,17 +24,20 @@ FAILED = "FAILED"
 QUERY_SHOW_DATABASES = """show databases;"""
 QUERY_SHOW_TABLES = """show tables in {schema_name};"""
 QUERY_TABLE_RECORDS_WITH_LIMIT = """select * from {schema_name}.{table_name} limit {limit};"""
-QUERY_TABLE_RECORDS = """select * from {schema_name}.{table_name} limit 1000;"""
+QUERY_TABLE_RECORDS = """select * from {schema_name}.{table_name} limit 100;"""
 
 SERVICE_URL = f"http://{HOST_IP}:{PORT}"
 # Athena services
+API_VALIDATE_USER = "/validate_user"
 API_GET_ATHENA_DATABASES = "/get_athena_databases"
 API_GET_ATHENA_TABLES = "/get_athena_tables"
 API_QUERY_ATHENA_TABLE = "/query_athena_table"
 API_RUN_QUERY_ON_ATHENA = "/run_query_on_athena"
+
 HEADERS = {
     'content-type': "application/json",
-    'cache-control': "no-cache",
+    'user-agent': "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
+                  "Chrome/89.0.4389.90 Safari/537.36"
 }
 
 
@@ -46,33 +46,55 @@ HEADERS = {
 ################################################################################
 
 class AthenaClient:
+    # Environment Variables
     _USER_NAME: str = 'USER_NAME'
     _ACCESS_TOKEN: str = 'ACCESS_TOKEN'
+    _USER_ROLE: str = 'USER_ROLE'
     _CCF_SERVICES_HOST: str = 'CCF_SERVICES_HOST'
 
     def __init__(self):
-        self._user_name = os.getenv(self._USER_NAME, None) or 'Karthik'
-        self._access_token = os.getenv(self._ACCESS_TOKEN, None) or '1'
+        self._user_name = os.getenv(self._USER_NAME, None)
+        self._access_token = os.getenv(self._ACCESS_TOKEN, None)
+        self._user_role = os.getenv(self._USER_ROLE, None)
         self._ccf_services_host = os.getenv(self._CCF_SERVICES_HOST, None) or SERVICE_URL
 
     def connect(self):
         # TODO - Add user validation
-        return "You're now connected..."
+        try:
+            payload = json.dumps({"user_name": self._user_name, "role_id": self._user_role})
+            r = requests.post(self._ccf_services_host + API_VALIDATE_USER, data=payload, headers=HEADERS,
+                              cookie=dict(sessionId=self._access_token))
+            response = r.json()
+            if response[STATUS] == SUCCESS:
+                return response[RESULT]
+            else:
+                return response[ERROR]
+
+        except requests.exceptions.HTTPError:
+            raise Exception("Http error occurred") from None
+        except requests.exceptions.ConnectionError:
+            raise ConnectionError("Connection error occurred") from None
+        except requests.exceptions.Timeout:
+            raise TimeoutError("Request timed out...") from None
+        except requests.exceptions.RequestException:
+            raise Exception("RequestException occurred") from None
 
     def databases(self, result_as_df=True):
         """Returns all the schemas available in Athena"""
         try:
-            payload = json.dumps({"user_name": self._user_name, "password": self._access_token})
-            r = requests.post(self._ccf_services_host + API_GET_ATHENA_DATABASES, data=payload, headers=HEADERS)
+            payload = json.dumps({"user_name": self._user_name, "role_id": self._user_role})
+            r = requests.post(self._ccf_services_host + API_GET_ATHENA_DATABASES, data=payload, headers=HEADERS,
+                              cookie=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df:
+                if result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT])
                     return response_as_df
                 else:
                     return response[RESULT]
             else:
-                return response[RESULT]
+                return response[ERROR]
+
         except requests.exceptions.HTTPError:
             raise Exception("Http error occurred") from None
         except requests.exceptions.ConnectionError:
@@ -85,17 +107,19 @@ class AthenaClient:
     def tables(self, schema_name, result_as_df=True):
         """Returns all the tables present in a schema"""
         try:
-            payload = json.dumps({"user_name": self._user_name, "password": self._access_token, "schema_name": schema_name})
-            r = requests.post(self._ccf_services_host + API_GET_ATHENA_TABLES, data=payload, headers=HEADERS)
+            payload = json.dumps({"user_name": self._user_name, "role_id": self._user_role, "schema_name": schema_name})
+            r = requests.post(self._ccf_services_host + API_GET_ATHENA_TABLES, data=payload, headers=HEADERS,
+                              cookie=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df:
+                if result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT])
                     return response_as_df
                 else:
                     return response[RESULT]
             else:
-                return response[RESULT]
+                return response[ERROR]
+
         except requests.exceptions.HTTPError:
             raise Exception("Http error occurred") from None
         except requests.exceptions.ConnectionError:
@@ -108,18 +132,20 @@ class AthenaClient:
     def top(self, schema_name, table_name, limit=None, result_as_df=True):
         """Returns records from a table"""
         try:
-            payload = json.dumps({"user_name": self._user_name, "password": self._access_token, "schema_name": schema_name,
+            payload = json.dumps({"user_name": self._user_name, "role_id": self._user_role, "schema_name": schema_name,
                                   "table_name": table_name, "limit": limit})
-            r = requests.post(self._ccf_services_host + API_QUERY_ATHENA_TABLE, data=payload, headers=HEADERS)
+            r = requests.post(self._ccf_services_host + API_QUERY_ATHENA_TABLE, data=payload, headers=HEADERS,
+                              cookie=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df:
+                if result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT])
                     return response_as_df
                 else:
                     return response[RESULT]
             else:
-                return response[RESULT]
+                return response[ERROR]
+
         except requests.exceptions.HTTPError:
             raise Exception("Http error occurred") from None
         except requests.exceptions.ConnectionError:
@@ -133,17 +159,19 @@ class AthenaClient:
         """Run a specified query"""
         try:
             payload = json.dumps(
-                {"user_name": self._user_name, "password": self._access_token, "query_string": query_string})
-            r = requests.post(self._ccf_services_host + API_RUN_QUERY_ON_ATHENA, data=payload, headers=HEADERS)
+                {"user_name": self._user_name, "role_id": self._user_role, "query_string": query_string})
+            r = requests.post(self._ccf_services_host + API_RUN_QUERY_ON_ATHENA, data=payload, headers=HEADERS,
+                              cookie=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df:
+                if result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT])
                     return response_as_df
                 else:
                     return response[RESULT]
             else:
-                return response[RESULT]
+                return response[ERROR]
+
         except requests.exceptions.HTTPError:
             raise Exception("Http error occurred") from None
         except requests.exceptions.ConnectionError:
